/* SPL Visualization - Dewey Number vs. Time
** by Charlie Roberts 1.18.2008
**
** This Processing file creates a visualization of the SPL data for one hour. Each transaction is represented by
** the placement of a rectangle on a grid, with the x position of the rectangle representing which minute of the hour
** the transaction occurred in, and the y position representing the highest level Dewey category the item falls into.
**
*/

import proxml.*;      // imports the proxml library

XMLInOut xml;                           // creates a variable named 'xml' that will hold an XMLInOut object
boolean isXMLParsed = false;            // a flag that will let us know when the xml has finished loading and parsing
PFont font;

int deweyClassChildNumber = 11;        // the child number of the deweyClass element in the SPL data
int checkInTimeChildNumber = 5;        // the child number of the ckitime element in the SPL data

/* GRAPH STYLE INFORMATION */
int graphXPos = 100;                   // the x Position of the graph
int graphYPos = 50;
int graphWidth = 600;                  // graph width
int graphHeight = 400;


class DataPoint {                      // the class for our data points
  int deweyRange;                      // 0 - 9
  int checkInTime;                     // 0 - 59 (minutes)

  DataPoint(int dr, int ct) {
    deweyRange = dr;
    checkInTime = ct;
  }
}

DataPoint[] dataPoints = new DataPoint[0];      // crate an empty (for now) array that will store our data points once the XML is parsed.

void setup() {
  size(720,500);

  font = loadFont("HelveticaNeue-CondensedBold-16.vlw"); 

  xml = new XMLInOut(this);                                                              // creates an XMLInOut object and stores it in the 'xml' variable
  xml.loadElement("http://www.charlie-roberts.com/259/utfConvert_16to8.php?&year=2008&month=01&day=02&hour=14&inOut=in");    // loads the xml file into the XMLInOut object stored in the variable 'xml' 
}

void xmlEvent(XMLElement element) {  // this method is called when the main (or root) xml element is finished loading. 'element' represents the root xml element
  XMLElement[] transactions;         // create an array of XMLElements named 'e'
  XMLElement transaction;
  int i,j;
  transactions = element.getChildren();                   // get all the child elements of the root element and store them in the array we just created  
  for(i=0; i<transactions.length; i++) {                  // loop for as long as i is less than the length of the transactions array, with i starting at 0
    transaction = transactions[i];
    if(transaction.getChild(deweyClassChildNumber).hasChildren()) {                                        // make sure that this transaction has a dewey class number
      float deweyClassNumber = float(transaction.getChild(deweyClassChildNumber).firstChild().getText());  // get the dewey number and convert to a float
      int deweyClassNumber_floor = floor(deweyClassNumber / 100);                                          // divide the dewey number by 100 and round down to get a value between 0 - 9
      String checkInTime = transaction.getChild(checkInTimeChildNumber).firstChild().getText();            // get the check in time in the format (hour : minute : seconds)
      String[] checkInTime_split = split(checkInTime, ':');                                                // split the time into three parts, the : is the delimiter for the split
      int checkInMinute = int(checkInTime_split[1]);                                                       // get the check in minute, which is the second item in the array we created with the last line
      DataPoint dp = new DataPoint(deweyClassNumber_floor, checkInMinute);                                 // create a new DataPoint with the parsed data from this transaction
      dataPoints = (DataPoint[])append(dataPoints, dp);                                                    // add our new DataPoint to the dataPoints array
    }
  }
  isXMLParsed = true;  // the xml is finished loading; now draw the graphics in the draw loop
}

void draw() {
  background(255);
  fill(0xFF775555);
  textFont(font); 
  if(!isXMLParsed) {                                  // if the xml has not finished parsing...
    text("LOADING AND PARSING XML ...", 15, 30);      // ... notify the user
  }else{
    drawGrid();                                       // draw the grid and axis labels (a custom function made below)
    drawData();                                       // draw the datapoints
  }
}

void drawData() {
 noStroke();                                                  // don't draw a border around the datapoint rectanges
 for(int i=0; i < dataPoints.length; i++) {                   // loop through the datapoints array
   DataPoint dp = dataPoints[i];                              // get the current datapoint
   int x = graphXPos + (dp.checkInTime * (graphWidth / 60));  // set the x position for the rectangle we will draw (take width of graph, divide by 60 (number of minutes in an hour) and multiply by check in time
   int y = 410 - (dp.deweyRange * (graphHeight / 10));        // set the y position for the rectangle we will draw
   fill(0x22660000);                                          // set the fill color for the rectangle
   rect(x , y, 10, 40);                                       // draw the rectangle
 }
 noLoop();                                                    // don't loop anymore once the data is drawn to the screen
}

void drawGrid() {
  stroke(0xFFDDCCCC);                                         // set the stroke (border) color for future drawing method calls
  fill(255);                                                  // set the fill to white
  rect(graphXPos, graphYPos, graphWidth, graphHeight);        // draw a rectangle with a border where the data will go... we will draw the datapoints over the top of this... just drawn to get the border
  fill(0xFFBB9999);                                           // set the fill color for when we draw our text
  textFont(font);
  textAlign(RIGHT);                                           // align our text flush right with the x coordinate we give when we call the text() method
  for(int i=0; i < 10; i++) {                                 // this loops write the axis labels for the y axis
      text(str(i * 100) + " - " + str((i * 100) + 100), 85, 435 - (i*40));    // str() converts a number to a string
  }
  text("2:00", 127, 475);  // write y axis labels
  text("3:00", 702, 475);
  textAlign(LEFT);
  fill(0xFF775555);
  text("DEWEY # VS. TIME  ::  SEATTLE PUBLIC LIBRARY  ::  01.02.2008", 15, 30); // write title text
}
